/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2005 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "IconTexture.h"

#include <shlobj.h>

extern UINT (WINAPI *_PrivateExtractIcons)(LPCTSTR,int,int,int,HICON*,UINT*,UINT,UINT);

IconTexture::IconTexture()
{
	memoryDC = 0;
	memoryBitmap = 0;

	memoryHoverDC = 0;
	memoryHoverBitmap = 0;

	memoryPressedDC = 0;
	memoryPressedBitmap = 0;

	transparencyNeeded = false;

	useAlphaMap = false;

	mustUseAlpha = false;
	hovermustUseAlpha = false;
	pressedmustUseAlpha = false;
}

IconTexture::~IconTexture()
{
	if(memoryBitmap)
	{
		memoryBitmap = (HBITMAP) SelectObject(memoryDC, memoryBitmap);
		DeleteObject(memoryBitmap);
	}
	if(memoryDC)
		DeleteDC(memoryDC);

	if(memoryHoverBitmap)
	{
		memoryHoverBitmap = (HBITMAP) SelectObject(memoryHoverDC, memoryHoverBitmap);
		DeleteObject(memoryHoverBitmap);
	}
	if(memoryHoverDC)
		DeleteDC(memoryHoverDC);

	if(memoryPressedBitmap)
	{
		memoryPressedBitmap = (HBITMAP) SelectObject(memoryPressedDC, memoryPressedBitmap);
		DeleteObject(memoryPressedBitmap);
	}
	if(memoryPressedDC)
		DeleteDC(memoryPressedDC);
}

void IconTexture::configure(const string &uniqueKey, const string &groupKey, bool alphaMap)
{
	useAlphaMap = alphaMap;
	int ExtractIconSize = Getx2RCInt(uniqueKey, groupKey, "ExtractIconSize", 32, 16, 256);
	bool oldstyle = Getx2RCBool(uniqueKey, groupKey, "OldStyleIconExtraction");

	string imageFile = Getx2RCString(uniqueKey, groupKey, "Image", "");
	if ( !imageFile.empty() )
	{
		CString m_strPath = imageFile.c_str();
		m_strPath.MakeLower();

		HICON noalphaMapIcon = NULL;
		int maxIconSize = ExtractIconSize;

		if ( m_strPath.Find(".ico") != -1 )
		{
			noalphaMapIcon = m_IconExtractor.ExtractIconFromIcoEx(m_strPath, memoryBitmap, maxIconSize);
		}
		else
		{
			int location = 0;
			if ( !oldstyle && m_strPath.Find(",") != -1 )
			{
				location = atoi(m_strPath.Right(m_strPath.GetLength()-(m_strPath.ReverseFind(',')+1)));
				m_strPath = m_strPath.Left(m_strPath.ReverseFind(','));
			}

			if ( !oldstyle && (m_strPath.Find(".exe") != -1 || m_strPath.Find(".dll") != -1) )
			{
				noalphaMapIcon = m_IconExtractor.ExtractIconFromExeEx(m_strPath, location, memoryBitmap, maxIconSize);
			}
			else if ( !oldstyle && m_strPath.Find(".lnk") != -1 )
			{
				noalphaMapIcon = GetIconFromLnkTarget(m_strPath, ExtractIconSize, 0);	
			}
			else
			{
				SHFILEINFO shFileInfo;
				if ( oldstyle )
					SHGetFileInfo(imageFile.c_str(), NULL, &shFileInfo, sizeof(shFileInfo), SHGFI_ICON | (ExtractIconSize < 32 ? SHGFI_SMALLICON : 0) );
				else
					SHGetFileInfo(m_strPath, NULL, &shFileInfo, sizeof(shFileInfo), SHGFI_ICON | (ExtractIconSize < 32 ? SHGFI_SMALLICON : 0) );
					
				noalphaMapIcon = shFileInfo.hIcon;
			}
		}

		iconSize = maxIconSize;

		if ( memoryBitmap != NULL || noalphaMapIcon != NULL )
			CreateDCFromSource(memoryDC, memoryBitmap, uniqueKey, groupKey, 0, noalphaMapIcon);

		if ( noalphaMapIcon == NULL || IsOS( OS_XP ) )
			mustUseAlpha = true;
		if ( noalphaMapIcon != NULL )
			DestroyIcon( noalphaMapIcon );
	}

	imageFile = Getx2RCString(uniqueKey, groupKey, "HoverImage", "");

	if ( !imageFile.empty() )
	{
		CString m_strPath = imageFile.c_str();
		m_strPath.MakeLower();

		HICON noalphaMapIcon = NULL;
		int maxIconSize = ExtractIconSize;

		if ( m_strPath.Find(".ico") != -1 )
		{
			noalphaMapIcon = m_IconExtractor.ExtractIconFromIcoEx(m_strPath, memoryHoverBitmap, maxIconSize);
		}
		else
		{
			int location = 0;
			if ( !oldstyle && m_strPath.Find(",") != -1 )
			{
				location = atoi(m_strPath.Right(m_strPath.GetLength()-(m_strPath.ReverseFind(',')+1)));
				m_strPath = m_strPath.Left(m_strPath.ReverseFind(','));
			}

			if ( !oldstyle && (m_strPath.Find(".exe") != -1 || m_strPath.Find(".dll") != -1) )
			{
				noalphaMapIcon = m_IconExtractor.ExtractIconFromExeEx(m_strPath, location, memoryHoverBitmap, maxIconSize);
			}
			else if ( !oldstyle && m_strPath.Find(".lnk") != -1 )
			{
				noalphaMapIcon = GetIconFromLnkTarget(m_strPath, ExtractIconSize, 1);	
			}
			else
			{
				SHFILEINFO shFileInfo;
				if ( oldstyle )
					SHGetFileInfo(imageFile.c_str(), NULL, &shFileInfo, sizeof(shFileInfo), SHGFI_ICON | (ExtractIconSize < 32 ? SHGFI_SMALLICON : 0) );
				else
					SHGetFileInfo(m_strPath, NULL, &shFileInfo, sizeof(shFileInfo), SHGFI_ICON | (ExtractIconSize < 32 ? SHGFI_SMALLICON : 0) );
				
				noalphaMapIcon = shFileInfo.hIcon;
			}
		}
		
		hovericonSize = maxIconSize;
			
		if ( memoryHoverBitmap != NULL || noalphaMapIcon != NULL )
			CreateDCFromSource(memoryHoverDC, memoryHoverBitmap, uniqueKey, groupKey, 1, noalphaMapIcon);

		if ( noalphaMapIcon == NULL || IsOS( OS_XP ) )
			hovermustUseAlpha = true;
		if ( noalphaMapIcon != NULL )
			DestroyIcon( noalphaMapIcon );
	}

	imageFile = Getx2RCString(uniqueKey, groupKey, "PressedImage", "");

	if ( !imageFile.empty() )
	{
		CString m_strPath = imageFile.c_str();
		m_strPath.MakeLower();

		HICON noalphaMapIcon = NULL;
		int maxIconSize = ExtractIconSize;

		if ( m_strPath.Find(".ico") != -1 )
		{
			noalphaMapIcon = m_IconExtractor.ExtractIconFromIcoEx(m_strPath, memoryPressedBitmap, maxIconSize);
		}
		else
		{
			int location = 0;
			if ( !oldstyle && m_strPath.Find(",") != -1 )
			{
				location = atoi(m_strPath.Right(m_strPath.GetLength()-(m_strPath.ReverseFind(',')+1)));
				m_strPath = m_strPath.Left(m_strPath.ReverseFind(','));
			}

			if ( !oldstyle && (m_strPath.Find(".exe") != -1 || m_strPath.Find(".dll") != -1) )
			{
				noalphaMapIcon = m_IconExtractor.ExtractIconFromExeEx(m_strPath, location, memoryPressedBitmap, maxIconSize);
			}
			else if ( !oldstyle && m_strPath.Find(".lnk") != -1 )
			{
				noalphaMapIcon = GetIconFromLnkTarget(m_strPath, ExtractIconSize, 2);	
			}
			else
			{
				SHFILEINFO shFileInfo;
				if ( oldstyle )
					SHGetFileInfo(imageFile.c_str(), NULL, &shFileInfo, sizeof(shFileInfo), SHGFI_ICON | (ExtractIconSize < 32 ? SHGFI_SMALLICON : 0) );
				else
					SHGetFileInfo(m_strPath, NULL, &shFileInfo, sizeof(shFileInfo), SHGFI_ICON | (ExtractIconSize < 32 ? SHGFI_SMALLICON : 0) );
			
				noalphaMapIcon = shFileInfo.hIcon;
			}
		}
		
		pressediconSize = maxIconSize;
			
		if ( memoryPressedBitmap != NULL || noalphaMapIcon != NULL )
			CreateDCFromSource(memoryPressedDC, memoryPressedBitmap, uniqueKey, groupKey, 2, noalphaMapIcon);

		if ( noalphaMapIcon == NULL || IsOS( OS_XP ) )
			pressedmustUseAlpha = true;
		if ( noalphaMapIcon != NULL )
			DestroyIcon( noalphaMapIcon );
	}
}

void IconTexture::apply(HDC hDC, int x, int y, int width, int height, int transparencyMode, int imageType)
{
	if ( imageType == 2 && (pressedmustUseAlpha && !useAlphaMap) )
	{
		applyOverlay(hDC, x, y, width, height, 255, 2);
		return;
	}
	else if ( imageType == 1 && (hovermustUseAlpha && !useAlphaMap) )
	{
		applyOverlay(hDC, x, y, width, height, 255, 1);
		return;
	}
	else if (imageType == 0 && (mustUseAlpha && !useAlphaMap) )
	{
		applyOverlay(hDC, x, y, width, height, 255, 0);
		return;
	}

	if ( imageType == 2 && memoryPressedBitmap == 0 )
		imageType = 1;

	if ( imageType == 1 && memoryHoverBitmap == 0 )
		imageType = 0;

	if ( imageType == 0 && memoryBitmap == 0 )
		return;

	HDC bufferDC = CreateCompatibleDC(NULL);

	BITMAPINFO alpha;
	ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
	alpha.bmiHeader.biWidth=width;      // Set size you need
	alpha.bmiHeader.biHeight=height;    // Set size you need
	alpha.bmiHeader.biPlanes=1;
	alpha.bmiHeader.biBitCount=32;      // Can be 8, 16, 32 bpp or other number
	alpha.bmiHeader.biSizeImage=0;
	alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
	alpha.bmiHeader.biClrUsed= 0;
	alpha.bmiHeader.biClrImportant= 0;
	VOID *pvBits;

	HBITMAP bufferBitmap = CreateDIBSection( bufferDC,
												&alpha,
												DIB_RGB_COLORS,
												&pvBits,
												NULL,
												0 );

	HBITMAP oldBMP = (HBITMAP) SelectObject(bufferDC, bufferBitmap);

	SetStretchBltMode(bufferDC, STRETCH_DELETESCANS);

	if (imageType == 0)
	{
		MultiBlt(bufferDC, 0, 0, width, height,
					memoryDC, 0, 0, iconSize, iconSize,
					0, 0, 0, 0,
					MULTIBLT_STRETCH, SRCCOPY);
	}
	else if (imageType == 1)
	{
		MultiBlt(bufferDC, 0, 0, width, height,
					memoryHoverDC, 0, 0, iconSize, iconSize,
					0, 0, 0, 0,
					MULTIBLT_STRETCH, SRCCOPY);
	}
	else if (imageType == 2)
	{
		MultiBlt(bufferDC, 0, 0, width, height,
					memoryPressedDC, 0, 0, iconSize, iconSize,
					0, 0, 0, 0,
					MULTIBLT_STRETCH, SRCCOPY);
	}

	BITMAP bm;
	GetObject(bufferBitmap, sizeof(BITMAP), &bm);

	BYTE *pPixel = (LPBYTE)bm.bmBits;

	for (int j = 0; j < bm.bmHeight * bm.bmWidth; ++j)
	{
		//Magic Pink
		if ( pPixel[0] == 0 && pPixel[1] == 0 && pPixel[2] == 0 && pPixel[3] == 0 )
		{
			if ( !useAlphaMap )
			{
				pPixel[0] = 255;
				pPixel[1] = 0;
				pPixel[2] = 255;
				pPixel[3] = 0;
			}
			else
			{
				pPixel[3] = 1;
			}
		}
			
		pPixel+= 4;
	}

	if ( transparencyMode == 2 || transparencyMode == 3 )
	{
		TransparentBltLS(hDC, x, y, width, height,
							bufferDC, 0, 0,
							RGB(255, 0, 255));
	}
	else
	{
		BitBlt(hDC, x, y, width, height, bufferDC, 0, 0, SRCCOPY);
	}

	bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
	DeleteObject(bufferBitmap);
	DeleteDC(bufferDC);
}

void IconTexture::applyOverlay(HDC hDC, int x, int y, int width, int height, int alphaValue, int imageType)
{	
	if ( imageType == 2 && memoryPressedBitmap == 0 )
		imageType = 1;

	if ( imageType == 1 && memoryHoverBitmap == 0 )
		imageType = 0;

	if ( imageType == 0 && memoryBitmap == 0 )
		return;

	HDC bufferDC = CreateCompatibleDC(NULL);

	BITMAPINFO alpha;
	ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
	alpha.bmiHeader.biWidth=width;      // Set size you need
	alpha.bmiHeader.biHeight=height;    // Set size you need
	alpha.bmiHeader.biPlanes=1;
	alpha.bmiHeader.biBitCount=32;      // Can be 8, 16, 32 bpp or other number
	alpha.bmiHeader.biSizeImage=0;
	alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
	alpha.bmiHeader.biClrUsed= 0;
	alpha.bmiHeader.biClrImportant= 0;
	VOID *pvBits;

	HBITMAP bufferBitmap = CreateDIBSection( bufferDC,
												&alpha,
												DIB_RGB_COLORS,
												&pvBits,
												NULL,
												0 );

	HBITMAP oldBMP = (HBITMAP) SelectObject(bufferDC, bufferBitmap);

	SetStretchBltMode(bufferDC, STRETCH_DELETESCANS);

	if (imageType == 0)
	{
		MultiBlt(bufferDC, 0, 0, width, height,
					memoryDC, 0, 0, iconSize, iconSize,
					0, 0, 0, 0,
					MULTIBLT_STRETCH, SRCCOPY);
	}
	else if (imageType == 1)
	{
		MultiBlt(bufferDC, 0, 0, width, height,
					memoryHoverDC, 0, 0, iconSize, iconSize,
					0, 0, 0, 0,
					MULTIBLT_STRETCH, SRCCOPY);
	}
	else if (imageType == 2)
	{
		MultiBlt(bufferDC, 0, 0, width, height,
					memoryPressedDC, 0, 0, iconSize, iconSize,
					0, 0, 0, 0,
					MULTIBLT_STRETCH, SRCCOPY);
	}

	BLENDFUNCTION blendFunction = { AC_SRC_OVER, 0, alphaValue, AC_SRC_ALPHA };

	AlphaBlend(hDC, x, y, width, height, bufferDC, 0, 0, width, height, blendFunction);

	bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
	DeleteObject(bufferBitmap);
	DeleteDC(bufferDC);
}

void IconTexture::CreateDCFromSource(HDC &destDC, HBITMAP &destBitmap, const string &uniqueKey, const string &groupKey, int imageType, HICON noalphaMapIcon)
{
	//Paint Saturation and Hueing Effects in MemoryDC's
	int _saturation = 255;
	COLORREF _hueColor = RGB(0,0,0);
	int _hueIntensity = 0;
	int _iconSize = 32;

	if (imageType == 0)
	{
		_saturation = Getx2RCInt(uniqueKey, groupKey, "SaturationIntensity", 255, 0, 255);
		_hueColor = Getx2RCColor(uniqueKey, groupKey, "HueColor", RGB(0, 0, 0));
		_hueIntensity = Getx2RCInt(uniqueKey, groupKey, "HueIntensity", 0, 0, 255);
		_iconSize = iconSize;
	}
	else if (imageType == 1)
	{
		_saturation = Getx2RCInt(uniqueKey, groupKey, "HoverSaturationIntensity", 255, 0, 255);
		_hueColor = Getx2RCColor(uniqueKey, groupKey, "HoverHueColor", RGB(0, 0, 0));
		_hueIntensity = Getx2RCInt(uniqueKey, groupKey, "HoverHueIntensity", 0, 0, 255);
		_iconSize = hovericonSize;
	}
	else if (imageType == 2)
	{
		_saturation = Getx2RCInt(uniqueKey, groupKey, "PressedSaturationIntensity", 255, 0, 255);
		_hueColor = Getx2RCColor(uniqueKey, groupKey, "PressedHueColor", RGB(0, 0, 0));
		_hueIntensity = Getx2RCInt(uniqueKey, groupKey, "PressedHueIntensity", 0, 0, 255);
		_iconSize = pressediconSize;
	}

	destDC = CreateCompatibleDC(NULL);

	VOID *pvBits;
	if ( noalphaMapIcon != NULL && destBitmap == NULL )
	{
		BITMAPINFO alpha;
		ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
		alpha.bmiHeader.biWidth=_iconSize;      // Set size you need
		alpha.bmiHeader.biHeight=_iconSize;    // Set size you need
		alpha.bmiHeader.biPlanes=1;
		alpha.bmiHeader.biBitCount=32;      // Can be 8, 16, 32 bpp or other number
		alpha.bmiHeader.biSizeImage=0;
		alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
		alpha.bmiHeader.biClrUsed= 0;
		alpha.bmiHeader.biClrImportant= 0;;

		destBitmap = CreateDIBSection( destDC,
										&alpha,
										DIB_RGB_COLORS,
										&pvBits,
										NULL,
										0 );
	}

	SelectObject(destDC, destBitmap);

	if ( noalphaMapIcon != NULL )
	{
		if ( !IsOS( OS_XP ) )
		{
			HBRUSH bgBrush = CreateSolidBrush(RGB(255,0,255));
			DrawIconEx(destDC, 0, 0, noalphaMapIcon, _iconSize, _iconSize, 0, bgBrush, DI_NORMAL);
			DeleteObject(bgBrush);
		}
		else
		{
			// set source alpha to zero
			for (int iy = 0; iy < _iconSize; iy++)
				for (int ix = 0; ix < _iconSize; ix++)
					((UINT32 *)pvBits)[ix + iy * _iconSize] = 0x00000000; 

			DrawIconEx(destDC, 0, 0, noalphaMapIcon, _iconSize, _iconSize, 0, NULL, DI_NORMAL);
		}
	}

	BITMAP bm;
	GetObject(destBitmap, sizeof(BITMAP), &bm);

	BYTE *pPixel = (LPBYTE)bm.bmBits;

	BYTE r = 0;
	BYTE g = 0;
	BYTE b = 0;

	for (int j = 0; j < bm.bmHeight * bm.bmWidth; ++j)
	{
		//Magic Pink and PureTransparency
		if ( pPixel[0] == 255 && pPixel[1] == 0 && pPixel[2] == 255 )
		{
			pPixel[0] = 0;
			pPixel[1] = 0;
			pPixel[2] = 0;
			pPixel[3] = 0;

			transparencyNeeded = true;

			pPixel+= 4;
			continue;
		}
				
		r = pPixel[0];
		g = pPixel[1];
		b = pPixel[2];

		//AlphaMap Colorizing
		//Saturation Effect
		if( _saturation != 255 && !(pPixel[0] == 0 && pPixel[1] == 0 && pPixel[2] == 0))
		{
			BYTE gray = (BYTE)(r * 0.3086 + g * 0.6094 + b * 0.0820);
			r = (BYTE)((r * _saturation + gray * (255 - _saturation) + 255) >> 8);
			g = (BYTE)((g * _saturation + gray * (255 - _saturation) + 255) >> 8);
			b = (BYTE)((b * _saturation + gray * (255 - _saturation) + 255) >> 8);
		}
		// Hue effect
		if( _hueIntensity != 0 && !(pPixel[0] == 0 && pPixel[1] == 0 && pPixel[2] == 0))
		{
			// The B & R values of the hue is swapped here, can somebody tell me why it only works that way?
			// test normal
			b = (BYTE)((r * (255 - _hueIntensity) + GetRValue(_hueColor) * _hueIntensity + 255) >> 8);
			g = (BYTE)((g * (255 - _hueIntensity) + GetGValue(_hueColor) * _hueIntensity + 255) >> 8);
			r = (BYTE)((b * (255 - _hueIntensity) + GetBValue(_hueColor) * _hueIntensity + 255) >> 8);
		}

		pPixel[0] = r;
		pPixel[1] = g;
		pPixel[2] = b;
		if ( !IsOS( OS_XP ) && noalphaMapIcon != NULL )
			pPixel[3] = 255;

		pPixel+= 4;
	}
}

HICON IconTexture::GetIconFromLnkTarget(CString path, int geticonSize, int imageMode)
{
	HICON resultIcon = NULL;

	if ( !IsOS( OS_XP ) )
	{
		// Special case for link files
		// If the icon is set for the link, retrieve it
		SHFILEINFO shfi;
		SHGetFileInfo(path, 0, &shfi, sizeof(SHFILEINFO), SHGFI_ICONLOCATION);
		// Extract the icon
		if ( _PrivateExtractIcons != NULL )
			_PrivateExtractIcons(shfi.szDisplayName, shfi.iIcon, (geticonSize >= 32 ? 32 : 16), (geticonSize >= 32 ? 32 : 16), &resultIcon, NULL, 1, NULL);
	}

	// If the icon was not explicitly set in the linkfile, we'll use the icon from the target file
	if(resultIcon == NULL)
	{
		HRESULT hRes;				
		char szInfo[1024] = {0};
		IShellLink* psl;

		// Create the ShellLink object
		hRes = CoCreateInstance(CLSID_ShellLink, NULL, CLSCTX_INPROC_SERVER, IID_IShellLink, (LPVOID*)&psl);
		if(SUCCEEDED(hRes))
		{
			IPersistFile* ppf;
			// Bind the ShellLink object to the Persistent File
			hRes = psl->QueryInterface(IID_IPersistFile, (LPVOID *) &ppf);
			if(SUCCEEDED(hRes))
			{
				WORD wsz[MAX_PATH];

				MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, path, -1, wsz, MAX_PATH);
				hRes = ppf->Load(wsz, 0);
							
				if(SUCCEEDED(hRes))
				{
					// Retrieve the target of the link file
					psl->GetPath(szInfo, 1024, NULL, SLGP_UNCPRIORITY);
				}
			}
		}
		psl->Release();

		// Extract the icon from the link target
		if ( imageMode == 0 )
			resultIcon = m_IconExtractor.ExtractIconFromExeEx(szInfo, 0, memoryBitmap, geticonSize);
		else if (imageMode == 1 )
			resultIcon = m_IconExtractor.ExtractIconFromExeEx(szInfo, 0, memoryHoverBitmap, geticonSize);
		else if (imageMode == 2 )
			resultIcon = m_IconExtractor.ExtractIconFromExeEx(szInfo, 0, memoryPressedBitmap, geticonSize);
	}

	return resultIcon;	
}
