/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2005 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "StdAfx.h"
#include "ImageTexture.h"

ImageTexture::ImageTexture()
{
	memoryDC = 0;
	memoryBitmap = 0;

	memoryHoverDC = 0;
	memoryHoverBitmap = 0;

	memoryPressedDC = 0;
	memoryPressedBitmap = 0;

	memoryAnimDC = 0;
	memoryAnimBitmap = 0; 

	memoryHoverAnimDC = 0;
	memoryHoverAnimBitmap = 0; 

	imageWidth = 0;
	imageHeight = 0;
	hoverimageWidth = 0;
	hoverimageHeight = 0;
	pressedimageWidth = 0;
	pressedimageHeight = 0;
	
	frameWidth = 0;
	frameHeight = 0;
	hoverframeWidth = 0;
	hoverframeHeight = 0;

	transparencyNeeded = false;

	useAlphaMap = false;
}

ImageTexture::~ImageTexture()
{
	if(memoryBitmap)
	{
		memoryBitmap = (HBITMAP) SelectObject(memoryDC, memoryBitmap);
		DeleteObject(memoryBitmap);
	}
	if(memoryDC)
		DeleteDC(memoryDC);

	if(memoryHoverBitmap)
	{
		memoryHoverBitmap = (HBITMAP) SelectObject(memoryHoverDC, memoryHoverBitmap);
		DeleteObject(memoryHoverBitmap);
	}
	if(memoryHoverDC)
		DeleteDC(memoryHoverDC);

	if(memoryPressedBitmap)
	{
		memoryPressedBitmap = (HBITMAP) SelectObject(memoryPressedDC, memoryPressedBitmap);
		DeleteObject(memoryPressedBitmap);
	}
	if(memoryPressedDC)
		DeleteDC(memoryPressedDC);

	if(memoryAnimBitmap)
	{
		memoryAnimBitmap = (HBITMAP) SelectObject(memoryAnimDC, memoryAnimBitmap);
		DeleteObject(memoryAnimBitmap);
	}
	if(memoryAnimDC)
		DeleteDC(memoryAnimDC);

	if(memoryHoverAnimBitmap)
	{
		memoryHoverAnimBitmap = (HBITMAP) SelectObject(memoryHoverAnimDC, memoryHoverAnimBitmap);
		DeleteObject(memoryHoverAnimBitmap);
	}
	if(memoryHoverAnimDC)
		DeleteDC(memoryHoverAnimDC);
}

void ImageTexture::configure(const string &uniqueKey, const string &groupKey, bool alphaMap)
{
	NameValuePair modeConstants[] =
	{
		{ "stretch", MULTIBLT_STRETCH },
		{ "tile", MULTIBLT_TILE },
		{ "tile-horizontal", MULTIBLT_TILEHORIZONTAL },
		{ "tile-vertical", MULTIBLT_TILEVERTICAL },
		{ "tilehorizontal", MULTIBLT_TILEHORIZONTAL },
		{ "tilevertical", MULTIBLT_TILEVERTICAL },
		{ 0, 0 }
	};

	useAlphaMap = alphaMap;

	saturation = Getx2RCInt(uniqueKey, groupKey, "SaturationIntensity", 255, 0, 255);
	hueIntensity = Getx2RCInt(uniqueKey, groupKey, "HueIntensity", 0, 0, 255);
	hueColor = Getx2RCColor(uniqueKey, groupKey, "HueColor", RGB(0, 0, 0));

	hoversaturation = Getx2RCInt(uniqueKey, groupKey, "HoverSaturationIntensity", 255, 0, 255);
	hoverhueIntensity = Getx2RCInt(uniqueKey, groupKey, "HoverHueIntensity", 0, 0, 255);
	hoverhueColor = Getx2RCColor(uniqueKey, groupKey, "HoverHueColor", hueColor);

	pressedsaturation = Getx2RCInt(uniqueKey, groupKey, "PressedSaturationIntensity", 255, 0, 255);
	pressedhueIntensity = Getx2RCInt(uniqueKey, groupKey, "PressedHueIntensity", 0, 0, 255);
	pressedhueColor = Getx2RCColor(uniqueKey, groupKey, "PressedHueColor", hoverhueColor);

	string imageFile = Getx2RCString(uniqueKey, groupKey, "Image", "");

	if ( !imageFile.empty() )
	{
		memoryBitmap = LoadLSImage(imageFile.c_str(), 0);

		if ( memoryBitmap != NULL )
		{
			if (useAlphaMap)
			{
				BITMAP bm;
				GetObject(memoryBitmap, sizeof(BITMAP), &bm);

				if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
					PrepareAlphaChannel(memoryBitmap, 0);
			}

			GetLSBitmapSize(memoryBitmap, &imageWidth, &imageHeight);

			memoryDC = CreateCompatibleDC(NULL);
			SelectObject(memoryDC, memoryBitmap);
		}
		else
		{
			transparencyNeeded = true;
		}
	}
	else
	{
		transparencyNeeded = true;
	}

	imageFile = Getx2RCString(uniqueKey, groupKey, "HoverImage", "");
	if (!imageFile.empty())
	{
		memoryHoverBitmap = LoadLSImage(imageFile.c_str(), 0);
		
		if ( memoryHoverBitmap != NULL )
		{
			if (useAlphaMap)
			{
				BITMAP bm;
				GetObject(memoryHoverBitmap, sizeof(BITMAP), &bm);

				if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
					PrepareAlphaChannel(memoryHoverBitmap, 1);
			}
				
			GetLSBitmapSize(memoryHoverBitmap, &hoverimageWidth, &hoverimageHeight);

			memoryHoverDC = CreateCompatibleDC(NULL);
			SelectObject(memoryHoverDC, memoryHoverBitmap);
		}
	}

	imageFile = Getx2RCString(uniqueKey, groupKey, "PressedImage", "");
	if (!imageFile.empty())
	{
		memoryPressedBitmap = LoadLSImage(imageFile.c_str(), 0);
		
		if ( memoryPressedBitmap != NULL )
		{
			if (useAlphaMap)
			{
				BITMAP bm;
				GetObject(memoryPressedBitmap, sizeof(BITMAP), &bm);

				if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
					PrepareAlphaChannel(memoryPressedBitmap, 2);
			}
				
			GetLSBitmapSize(memoryPressedBitmap, &pressedimageWidth, &pressedimageHeight);

			memoryPressedDC = CreateCompatibleDC(NULL);
			SelectObject(memoryPressedDC, memoryPressedBitmap);
		}
	}

	leftEdge = Getx2RCInt(uniqueKey, groupKey, "ImageLeftEdge", 0, 0);
	topEdge = Getx2RCInt(uniqueKey, groupKey, "ImageTopEdge", 0, 0);
	rightEdge = Getx2RCInt(uniqueKey, groupKey, "ImageRightEdge", 0, 0);
	bottomEdge = Getx2RCInt(uniqueKey, groupKey, "ImageBottomEdge", 0, 0);

	mode = Getx2RCNamedValue(uniqueKey, groupKey, "ImageMode", modeConstants, MULTIBLT_STRETCH);

	hoverleftEdge = Getx2RCInt(uniqueKey, groupKey, "HoverImageLeftEdge", leftEdge, 0);
	hovertopEdge = Getx2RCInt(uniqueKey, groupKey, "HoverImageTopEdge", topEdge, 0);
	hoverrightEdge = Getx2RCInt(uniqueKey, groupKey, "HoverImageRightEdge", rightEdge, 0);
	hoverbottomEdge = Getx2RCInt(uniqueKey, groupKey, "HoverImageBottomEdge", bottomEdge, 0);

	hovermode = Getx2RCNamedValue(uniqueKey, groupKey, "HoverImageMode", modeConstants, mode);

	pressedleftEdge = Getx2RCInt(uniqueKey, groupKey, "PressedImageLeftEdge", hoverleftEdge, 0);
	pressedtopEdge = Getx2RCInt(uniqueKey, groupKey, "PressedImageTopEdge", hovertopEdge, 0);
	pressedrightEdge = Getx2RCInt(uniqueKey, groupKey, "PressedImageRightEdge", hoverrightEdge, 0);
	pressedbottomEdge = Getx2RCInt(uniqueKey, groupKey, "PressedImageBottomEdge", hoverbottomEdge, 0);

	pressedmode = Getx2RCNamedValue(uniqueKey, groupKey, "PressedImageMode", modeConstants, hovermode);
	
	int animimageWidth = 0;
	int animimageHeight = 0;
	imageFile = GetxRCString(uniqueKey, "Frames", "");
	if (!imageFile.empty())
	{
		memoryAnimBitmap = LoadLSImage(imageFile.c_str(), 0);

		if ( memoryAnimBitmap != NULL )
		{
			if (useAlphaMap)
			{
				BITMAP bm;
				GetObject(memoryAnimBitmap, sizeof(BITMAP), &bm);

				if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
					PrepareAlphaChannel(memoryAnimBitmap, 0);
			}

			GetLSBitmapSize(memoryAnimBitmap, &animimageWidth, &animimageHeight);

			memoryAnimDC = CreateCompatibleDC(NULL);
			SelectObject(memoryAnimDC, memoryAnimBitmap);

			frameWidth = GetxRCInt(uniqueKey, "FrameWidth", 0);
			frameHeight = GetxRCInt(uniqueKey, "FrameHeight", 0);
		}
	}

	int hoveranimimageWidth = 0;
	int hoveranimimageHeight = 0;
	imageFile = GetxRCString(uniqueKey, "HoverFrames", "");
	if (!imageFile.empty())
	{
		memoryHoverAnimBitmap = LoadLSImage( imageFile.c_str(), 0 );

		if ( memoryHoverAnimBitmap != NULL )
		{
			if (useAlphaMap)
			{
				BITMAP bm;
				GetObject(memoryHoverAnimBitmap, sizeof(BITMAP), &bm);

				if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
					PrepareAlphaChannel(memoryHoverAnimBitmap, 1);
			}
			
			GetLSBitmapSize(memoryHoverAnimBitmap, &hoveranimimageWidth, &hoveranimimageHeight);
				
			memoryHoverAnimDC = CreateCompatibleDC(NULL);
			SelectObject(memoryHoverAnimDC, memoryHoverAnimBitmap);

			hoverframeWidth = GetxRCInt(uniqueKey, "HoverFrameWidth", 0);
			hoverframeHeight = GetxRCInt(uniqueKey, "HoverFrameHeight", 0);
		}
	}
		
	if (memoryDC && memoryBitmap && !useAlphaMap)
		paintEffects(memoryDC, imageWidth, imageHeight, 0);
	if (memoryHoverDC && memoryHoverBitmap && !useAlphaMap)
		paintEffects(memoryHoverDC, hoverimageWidth, hoverimageHeight, 1);
	if (memoryPressedDC && memoryPressedBitmap && !useAlphaMap)
		paintEffects(memoryPressedDC, pressedimageWidth, pressedimageHeight, 2);
	if (memoryAnimDC && memoryAnimBitmap && !useAlphaMap)
		paintEffects(memoryAnimDC, animimageWidth, animimageHeight, 0);
	if (memoryHoverAnimDC && memoryHoverAnimBitmap && !useAlphaMap)
		paintEffects(memoryHoverAnimDC, hoveranimimageWidth, hoveranimimageHeight, 1);
}

void ImageTexture::apply(HDC hDC, int x, int y, int width, int height, int transparencyMode, int imageType)
{
	if ( imageType == 2 && memoryPressedBitmap == 0 )
		imageType = 1;

	if ( imageType == 1 && memoryHoverBitmap == 0 )
		imageType = 0;

	if ( imageType == 0 && memoryBitmap == 0 )
		return;

	if (transparencyMode == 2 || transparencyMode == 3)
	{
		HDC bufferDC = CreateCompatibleDC(hDC);
		HBITMAP bufferBitmap = CreateCompatibleBitmap(hDC, width, height);
		HBITMAP oldBMP = (HBITMAP) SelectObject(bufferDC, bufferBitmap);

		SetStretchBltMode(bufferDC, STRETCH_DELETESCANS);

		if (imageType == 0)
		{
			MultiBlt(bufferDC, 0, 0, width, height,
						memoryDC, 0, 0, imageWidth, imageHeight,
						leftEdge, topEdge, rightEdge, bottomEdge,
						mode, SRCCOPY);
		}
		else if (imageType == 1)
		{
			MultiBlt(bufferDC, 0, 0, width, height,
						memoryHoverDC, 0, 0, hoverimageWidth, hoverimageHeight,
						hoverleftEdge, hovertopEdge, hoverrightEdge, hoverbottomEdge,
						hovermode, SRCCOPY);
		}
		else if (imageType == 2)
		{
			MultiBlt(bufferDC, 0, 0, width, height,
						memoryPressedDC, 0, 0, pressedimageWidth, pressedimageHeight,
						pressedleftEdge, pressedtopEdge, pressedrightEdge, pressedbottomEdge,
						pressedmode, SRCCOPY);
		}
			
		TransparentBltLS(hDC, x, y, width, height,
							bufferDC, 0, 0,
							RGB(255, 0, 255));

		bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
		DeleteObject(bufferBitmap);
		DeleteDC(bufferDC);
	}
	else
	{
		SetStretchBltMode(hDC, STRETCH_DELETESCANS);

		if (imageType == 0)
		{
			MultiBlt(hDC, x, y, width, height,
						memoryDC, 0, 0, imageWidth, imageHeight,
						leftEdge, topEdge, rightEdge, bottomEdge,
						mode, SRCCOPY);
		}
		else if (imageType == 1)
		{
			MultiBlt(hDC, x, y, width, height,
						memoryHoverDC, 0, 0, hoverimageWidth, hoverimageHeight,
						hoverleftEdge, hovertopEdge, hoverrightEdge, hoverbottomEdge,
						hovermode, SRCCOPY);
		}
		else if (imageType == 2)
		{
			MultiBlt(hDC, x, y, width, height,
						memoryPressedDC, 0, 0, pressedimageWidth, pressedimageHeight,
						pressedleftEdge, pressedtopEdge, pressedrightEdge, pressedbottomEdge,
						pressedmode, SRCCOPY);
		}
	}
}

void ImageTexture::applyOverlay(HDC hDC, int x, int y, int width, int height, int alphaValue, int imageType)
{
	if ( imageType == 2 && memoryPressedBitmap == 0 )
		imageType = 1;

	if ( imageType == 1 && memoryHoverBitmap == 0 )
		imageType = 0;

	if ( imageType == 0 && memoryBitmap == 0 )
		return;

	HDC bufferDC = CreateCompatibleDC(NULL);

	BITMAPINFO alpha;
	ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
	alpha.bmiHeader.biWidth=width;      // Set size you need
	alpha.bmiHeader.biHeight=height;    // Set size you need
	alpha.bmiHeader.biPlanes=1;
	alpha.bmiHeader.biBitCount=32;      // Can be 8, 16, 32 bpp or other number
	alpha.bmiHeader.biSizeImage=0;
	alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
	alpha.bmiHeader.biClrUsed= 0;
	alpha.bmiHeader.biClrImportant= 0;
	VOID *pvBits;

	HBITMAP bufferBitmap = CreateDIBSection( bufferDC,
												&alpha,
												DIB_RGB_COLORS,
												&pvBits,
												NULL,
												0 );

	HBITMAP oldBMP = (HBITMAP) SelectObject(bufferDC, bufferBitmap);

	// set source alpha to zero
    for (int iy = 0; iy < height; iy++)
        for (int ix = 0; ix < width; ix++)
            ((UINT32 *)pvBits)[ix + iy * width] = 0x00000000; 

	SetStretchBltMode(bufferDC, STRETCH_DELETESCANS);

	if (imageType == 0)
	{
		MultiBlt(bufferDC, 0, 0, width, height,
					memoryDC, 0, 0, imageWidth, imageHeight,
					leftEdge, topEdge, rightEdge, bottomEdge,
					mode, SRCCOPY);
	}
	else if (imageType == 1)
	{
		MultiBlt(bufferDC, 0, 0, width, height,
					memoryHoverDC, 0, 0, hoverimageWidth, hoverimageHeight,
					hoverleftEdge, hovertopEdge, hoverrightEdge, hoverbottomEdge,
					hovermode, SRCCOPY);
	}
	else if (imageType == 2)
	{
		MultiBlt(bufferDC, 0, 0, width, height,
					memoryPressedDC, 0, 0, pressedimageWidth, pressedimageHeight,
					pressedleftEdge, pressedtopEdge, pressedrightEdge, pressedbottomEdge,
					pressedmode, SRCCOPY);
	}

	BITMAP bm;
	GetObject(bufferBitmap, sizeof(BITMAP), &bm);

	BYTE *pPixel = (LPBYTE)bm.bmBits;

	for (int j = 0; j < bm.bmHeight * bm.bmWidth; ++j)
	{
		//Remove Magic Pink
		if ( (pPixel[0] == 255 && pPixel[1] == 0 && pPixel[2] == 255) )
		{
			pPixel[0] = 0;
			pPixel[1] = 0;
			pPixel[2] = 0;
			pPixel[3] = 0;
		}
		else if ( !useAlphaMap )
		{
			pPixel[3] = 255;
		}

		pPixel+= 4;
	}
	
	BLENDFUNCTION blendFunction = { AC_SRC_OVER, 0, alphaValue, AC_SRC_ALPHA };

	AlphaBlend(hDC, x, y, width, height, bufferDC, 0, 0, width, height, blendFunction);

	bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
	DeleteObject(bufferBitmap);
	DeleteDC(bufferDC);
}

void ImageTexture::applyAnim(HDC hDC, int x, int y, int width, int height, int transparencyMode, int imageType, int frameCounter)
{
	if (imageType == 1 && memoryHoverAnimBitmap == 0)
		imageType = 0;
	
	if (imageType == 0 && memoryAnimBitmap == 0)
		return;

	if (transparencyMode == 2 || transparencyMode == 3)
	{
		HDC bufferDC = CreateCompatibleDC(hDC);
		HBITMAP bufferBitmap = CreateCompatibleBitmap(hDC, width, height);
		HBITMAP oldBMP = (HBITMAP) SelectObject(bufferDC, bufferBitmap);

		SetStretchBltMode(bufferDC, STRETCH_DELETESCANS);

		if (imageType == 0)
		{
			MultiBlt(bufferDC, 0, 0, width, height,
				memoryAnimDC, (frameCounter-1)*frameWidth, 0, frameWidth, frameHeight,
				leftEdge, topEdge, rightEdge, bottomEdge,
				mode, SRCCOPY);
		}
		else if (imageType == 1)
		{
			MultiBlt(bufferDC, 0, 0, width, height,
				memoryHoverAnimDC, (frameCounter-1)*hoverframeWidth, 0, hoverframeWidth, hoverframeHeight,
				hoverleftEdge, hovertopEdge, hoverrightEdge, hoverbottomEdge,
				hovermode, SRCCOPY);
		}

		TransparentBltLS(hDC, x, y, width, height,
			bufferDC, 0, 0,
			RGB(255, 0, 255));

		bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
		DeleteObject(bufferBitmap);
		DeleteDC(bufferDC);
	}
	else
	{
		SetStretchBltMode(hDC, STRETCH_DELETESCANS);

		if (imageType == 0)
		{
			MultiBlt(hDC, x, y, width, height,
					memoryAnimDC, (frameCounter-1)*frameWidth, 0, frameWidth, frameHeight,
					leftEdge, topEdge, rightEdge, bottomEdge,
					mode, SRCCOPY);
		}
		else if (imageType == 1)
		{
			MultiBlt(hDC, x, y, width, height,
					memoryHoverAnimDC, (frameCounter-1)*hoverframeWidth, 0, hoverframeWidth, hoverframeHeight,
					hoverleftEdge, hovertopEdge, hoverrightEdge, hoverbottomEdge,
					hovermode, SRCCOPY);
		}
	}
}

void ImageTexture::PrepareAlphaChannel(HBITMAP &bmp, int imageType)
{
	//Paint Saturation and Hueing Effects in MemoryDC's
	int _saturation = 255;
	COLORREF _hueColor = RGB(0,0,0);
	int _hueIntensity = 0;

	if (imageType == 0)
	{
		_saturation = saturation;
		_hueColor = hueColor;
		_hueIntensity = hueIntensity;
	}
	else if (imageType == 1)
	{
		_saturation = hoversaturation;
		_hueColor = hoverhueColor;
		_hueIntensity = hoverhueIntensity;
	}
	else if (imageType == 2)
	{
		_saturation = pressedsaturation;
		_hueColor = pressedhueColor;
		_hueIntensity = pressedhueIntensity;
	}

	BITMAP bm;
	GetObject(bmp, sizeof(BITMAP), &bm);

	BYTE *pPixel = (LPBYTE)bm.bmBits;

	BYTE r = 0;
	BYTE g = 0;
	BYTE b = 0;

	for (int i = 0; i < bm.bmHeight * bm.bmWidth; ++i)
	{
		//Magic Pink
		if (pPixel[0] == 255 && pPixel[1] == 0 && pPixel[2] == 255)
			pPixel[3] = 0;

		//Prepare Text
		if (pPixel[3] == 0)
			pPixel[3] = 1;

		r = pPixel[0];
		g = pPixel[1];
		b = pPixel[2];

		//AlphaMap Colorizing
		//Saturation Effect
		if( _saturation != 255 )
		{
			BYTE gray = (BYTE)(r * 0.3086 + g * 0.6094 + b * 0.0820);
			r = (BYTE)((r * _saturation + gray * (255 - _saturation) + 255) >> 8);
			g = (BYTE)((g * _saturation + gray * (255 - _saturation) + 255) >> 8);
			b = (BYTE)((b * _saturation + gray * (255 - _saturation) + 255) >> 8);
		}
		// Hue effect
		if( _hueIntensity != 0 )
		{
			// The B & R values of the hue is swapped here, can somebody tell me why it only works that way?
			// test normal
			b = (BYTE)((r * (255 - _hueIntensity) + GetRValue(_hueColor) * _hueIntensity + 255) >> 8);
			g = (BYTE)((g * (255 - _hueIntensity) + GetGValue(_hueColor) * _hueIntensity + 255) >> 8);
			r = (BYTE)((b * (255 - _hueIntensity) + GetBValue(_hueColor) * _hueIntensity + 255) >> 8);
		}
		
		//PreMultiply
		if ( pPixel[3] == 1 )
		{
			pPixel[0] = 0;
			pPixel[1] = 0;
			pPixel[2] = 0;
		}
		else
		{
			pPixel[0] = r * pPixel[3] / 255;
			pPixel[1] = g * pPixel[3] / 255;
			pPixel[2] = b * pPixel[3] / 255;
		}
	
		pPixel+= 4;
	}
}

void ImageTexture::paintEffects(HDC &destHDC, int width, int height, int imageType)
{
	//Paint Saturation and Hueing Effects in MemoryDC's
	int _saturation = 255;
	COLORREF _hueColor = RGB(0,0,0);
	int _hueIntensity = 0;

	if (imageType == 0)
	{
		_saturation = saturation;
		_hueColor = hueColor;
		_hueIntensity = hueIntensity;
	}
	else if (imageType == 1)
	{
		_saturation = hoversaturation;
		_hueColor = hoverhueColor;
		_hueIntensity = hoverhueIntensity;
	}
	else if (imageType == 2)
	{
		_saturation = pressedsaturation;
		_hueColor = pressedhueColor;
		_hueIntensity = pressedhueIntensity;
	}

	int iy = 0;
	int ix = 0;
	COLORREF cl = 0;
	BYTE r = 0;
	BYTE g = 0;
	BYTE b = 0;

	for(iy = 0; iy < height; iy++)
	{
		for(ix = 0; ix < width; ix++)
		{
			cl = GetPixel(destHDC, ix, iy);
			r = GetRValue(cl);
			g = GetGValue(cl);
			b = GetBValue(cl);
			

			if( !( r == 255 && g == 0 && b == 255 ) )
			{
				// Saturation effect
				if(_saturation != 255)
				{
					BYTE gray = (BYTE)(r * 0.3086 + g * 0.6094 + b * 0.0820);

					r = (BYTE)((r * _saturation + gray * (255 - _saturation) + 255) >> 8);
					g = (BYTE)((g * _saturation + gray * (255 - _saturation) + 255) >> 8);
					b = (BYTE)((b * _saturation + gray * (255 - _saturation) + 255) >> 8);
				}

				// Hue effect
				if(_hueIntensity != 0)
				{
					r = (BYTE)((r * (255 - _hueIntensity) + GetRValue(_hueColor) * _hueIntensity + 255) >> 8);
					g = (BYTE)((g * (255 - _hueIntensity) + GetGValue(_hueColor) * _hueIntensity + 255) >> 8);
					b = (BYTE)((b * (255 - _hueIntensity) + GetBValue(_hueColor) * _hueIntensity + 255) >> 8);
				}
			}
			else 
			{
				transparencyNeeded = true;
			}

			SetPixelV(destHDC, ix, iy, RGB(r, g, b));
		}
	}
	//End Saturation / Hueing
}