/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2005 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "StdAfx.h"
#include "SolidTexture.h"

SolidTexture::SolidTexture()
{
}

SolidTexture::~SolidTexture()
{
	Gradient.clear();
	HoverGradient.clear();
	PressedGradient.clear();
}

void SolidTexture::configure(const string &uniqueKey, const string &groupKey, bool alphaMap)
{
	string BevelCFG = "0";
	BevelCFG = Getx2RCLine(uniqueKey, groupKey, "SolidBevelSize", "0");
	parseBevelCFG(BevelCFG, bevelSizeLeft, bevelSizeRight, bevelSizeTop, bevelSizeBottom);
	
	BevelCFG = Getx2RCLine(uniqueKey, groupKey, "HoverSolidBevelSize", BevelCFG);
	parseBevelCFG(BevelCFG, hoverbevelSizeLeft, hoverbevelSizeRight, hoverbevelSizeTop, hoverbevelSizeBottom);

	BevelCFG = Getx2RCLine(uniqueKey, groupKey, "PressedSolidBevelSize", BevelCFG);
	parseBevelCFG(BevelCFG, pressedbevelSizeLeft, pressedbevelSizeRight, pressedbevelSizeTop, pressedbevelSizeBottom);

	string solidPaintingColors = Getx2RCLine(uniqueKey, groupKey, "SolidColors", "ffffff");
	parseCFG(solidPaintingColors, backgroundColor, lightBevelColor, darkBevelColor);
	//BG Insert in Gradient List
	Gradient.insert(Gradient.end(), backgroundColor );

	gradientColor = Getx2RCLine(uniqueKey, groupKey, "SolidGradientColors", "");
	parseGradientCFG( gradientColor, 0 );

	verticalGradient = Getx2RCBool(uniqueKey, groupKey, "SolidGradientVertical");

	repeatedGradient = Getx2RCInt(uniqueKey, groupKey, "SolidGradientRepeated", 0, 0);

	solidPaintingColors = Getx2RCLine(uniqueKey, groupKey, "HoverSolidColors", "");
	if (!solidPaintingColors.empty())
	{
		parseCFG(solidPaintingColors, hoverbackgroundColor, hoverlightBevelColor, hoverdarkBevelColor);
	}
	else
	{
		hoverbackgroundColor = backgroundColor;
		hoverdarkBevelColor = darkBevelColor;
		hoverlightBevelColor = lightBevelColor;
	}
	//BG Insert in Gradient List
	HoverGradient.insert(HoverGradient.end(), hoverbackgroundColor );

	hovergradientColor = Getx2RCLine(uniqueKey, groupKey, "HoverSolidGradientColors", gradientColor);
	parseGradientCFG( hovergradientColor, 1 );

	hoververticalGradient = Getx2RCBool(uniqueKey, groupKey, "HoverSolidGradientVertical", verticalGradient);

	repeatedHoverGradient = Getx2RCInt(uniqueKey, groupKey, "HoverSolidGradientRepeated", repeatedGradient, 0);

	solidPaintingColors = Getx2RCLine(uniqueKey, groupKey, "PressedSolidColors", "");
	if (!solidPaintingColors.empty())
	{
		parseCFG(solidPaintingColors, pressedbackgroundColor, pressedlightBevelColor, presseddarkBevelColor);
	}
	else
	{
		pressedbackgroundColor = hoverbackgroundColor;
		presseddarkBevelColor = hoverdarkBevelColor;
		pressedlightBevelColor = hoverlightBevelColor;
	}
	//BG Insert in Gradient List
	PressedGradient.insert(PressedGradient.end(), pressedbackgroundColor );

	pressedgradientColor = Getx2RCLine(uniqueKey, groupKey, "PressedSolidGradientColors", hovergradientColor);
	parseGradientCFG( pressedgradientColor, 2 );

	pressedverticalGradient = Getx2RCBool(uniqueKey, groupKey, "PressedSolidGradientVertical", hoververticalGradient);

	repeatedPressedGradient = Getx2RCInt(uniqueKey, groupKey, "PressedSolidGradientRepeated", repeatedHoverGradient, 0);
}

void SolidTexture::parseCFG(string cfg, COLORREF &bgcolor, COLORREF &lightcolor, COLORREF &darkcolor)
{
	while ( strstr( cfg.c_str(), "#" ) != NULL )
		cfg.replace(cfg.find("#"), 1, "", 0);
	
	int color;
	char main[32], light[32], dark[32];
	char *buffers[] = {main, light, dark};

	int numTokens = LCTokenize(cfg.c_str(), buffers, 3, NULL);
		
	if(numTokens == 1)
	{
		color = strtol(main, NULL, 16);
		bgcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		lightcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		darkcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
	}
	else if (numTokens == 2)
	{
		color = strtol(main, NULL, 16);
		bgcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		color = strtol(light, NULL, 16);
		lightcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		darkcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
	}
	else if (numTokens == 3)
	{
		color = strtol(main, NULL, 16);
		bgcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		color = strtol(light, NULL, 16);
		lightcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		color = strtol(dark, NULL, 16);
		darkcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
	}
}

void SolidTexture::parseGradientCFG(string cfg, int imageType)
{
	while ( strstr( cfg.c_str(), "#" ) != NULL )
		cfg.replace(cfg.find("#"), 1, "", 0);

	int color;
	COLORREF insertColor;
	char token[MAX_LINE_LENGTH];
	LPCSTR nextToken = cfg.c_str();
	
	int counter = 1;
	while (GetToken(nextToken, token, &nextToken, false))
	{
		color = strtol(token, NULL, 16);
		insertColor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		if (imageType == 0)
		{
			Gradient.insert(Gradient.end(), insertColor );
		}
		else if (imageType == 1)
		{
			HoverGradient.insert(HoverGradient.end(), insertColor );
		}
		else if (imageType == 2)
		{
			PressedGradient.insert(PressedGradient.end(), insertColor );
		}
		counter++;
	}
}

void SolidTexture::parseBevelCFG(string cfg, int &leftBevel, int &rightBevel, int &topBevel, int &bottomBevel )
{
	char left[32], right[32], top[32], bottom[32];
	char *buffers[] = { left, right, top, bottom };

	leftBevel = 0;
	rightBevel = 0;
	topBevel = 0;
	bottomBevel = 0;

	int numTokens = LCTokenize(cfg.c_str(), buffers, 4, NULL);
		
	if(numTokens == 1)
	{
		leftBevel = atoi(left);
		rightBevel = leftBevel;
		topBevel = leftBevel;
		bottomBevel = leftBevel;
	}
	else if (numTokens == 2)
	{
		leftBevel = atoi(left);
		rightBevel = leftBevel;
		topBevel = atoi(right);
		bottomBevel = topBevel;
	}
	else if (numTokens == 4)
	{
		leftBevel = atoi(left);
		rightBevel = atoi(right);
		topBevel = atoi(top);
		bottomBevel = atoi(bottom);
	}
}

void SolidTexture::apply(HDC hDC, int x, int y, int width, int height, int transparencyMode, int imageType)
{
	RECT r;
	r.left = x;
	r.top = y;
	r.right = r.left + width;
	r.bottom = r.top + height;

	int bevelWidth = 0;
	int bevelHeight = 0;

	if (imageType == 0)
	{
		bevelWidth = bevelSizeLeft+bevelSizeRight;
		bevelHeight = bevelSizeTop+bevelSizeBottom;
		
		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(hDC, darkBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-bevelSizeRight;
			r.bottom = r.bottom-bevelSizeBottom;
			::SetBkColor(hDC, lightBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+bevelSizeLeft;
			r.top = r.top+bevelSizeTop;

			if ( Gradient.size() > 1 )
			{
				solidGradientPaint(hDC, Gradient, r, imageType);
			}
			else
			{
				::SetBkColor(hDC, backgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( Gradient.size() > 1 )
			{
				solidGradientPaint(hDC, Gradient, r, imageType);
			}
			else
			{
				::SetBkColor(hDC, backgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if (imageType == 1)
	{
		bevelWidth = hoverbevelSizeLeft+hoverbevelSizeRight;
		bevelHeight = hoverbevelSizeTop+hoverbevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(hDC, hoverdarkBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-hoverbevelSizeRight;
			r.bottom = r.bottom-hoverbevelSizeBottom;
			::SetBkColor(hDC, hoverlightBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+hoverbevelSizeLeft;
			r.top = r.top+hoverbevelSizeTop;

			if ( HoverGradient.size() > 1 )
			{
				solidGradientPaint(hDC, HoverGradient, r, imageType);
			}
			else
			{
				::SetBkColor(hDC, hoverbackgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( HoverGradient.size() > 1 )
			{
				solidGradientPaint(hDC, HoverGradient, r, imageType);
			}
			else
			{
				::SetBkColor(hDC, hoverbackgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if (imageType == 2)
	{
		bevelWidth = pressedbevelSizeLeft+pressedbevelSizeRight;
		bevelHeight = pressedbevelSizeTop+pressedbevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(hDC, presseddarkBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-pressedbevelSizeRight;
			r.bottom = r.bottom-pressedbevelSizeBottom;
			::SetBkColor(hDC, pressedlightBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+pressedbevelSizeLeft;
			r.top = r.top+pressedbevelSizeTop;

			if ( PressedGradient.size() > 1 )
			{
				solidGradientPaint(hDC, PressedGradient, r, imageType);
			}
			else
			{
				::SetBkColor(hDC, pressedbackgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( PressedGradient.size() > 1 )
			{
				solidGradientPaint(hDC, PressedGradient, r, imageType);
			}
			else
			{
				::SetBkColor(hDC, pressedbackgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
}

void SolidTexture::applyOverlay(HDC hDC, int x, int y, int width, int height, int alphaValue, int imageType)
{
	HDC bufferDC = CreateCompatibleDC(NULL);

	BITMAPINFO alpha;
	ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
	alpha.bmiHeader.biWidth=width;      // Set size you need
	alpha.bmiHeader.biHeight=height;    // Set size you need
	alpha.bmiHeader.biPlanes=1;
	alpha.bmiHeader.biBitCount=32;            // Can be 8, 16, 32 bpp or other number
	alpha.bmiHeader.biSizeImage=0;
	alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
	alpha.bmiHeader.biClrUsed= 0;
	alpha.bmiHeader.biClrImportant= 0;
	VOID *pvBits;

	HBITMAP bufferBitmap = CreateDIBSection( bufferDC,
												&alpha,
												DIB_RGB_COLORS,
												&pvBits,
												NULL,
												0 );

	HBITMAP oldBMP = (HBITMAP) SelectObject(bufferDC, bufferBitmap);

	apply(bufferDC, 0, 0, width, height, 1, imageType);

	BITMAP bm;
	GetObject(bufferBitmap, sizeof(BITMAP), &bm);

	BYTE *pPixel = (LPBYTE)bm.bmBits;

	for (int j = 0; j < bm.bmHeight * bm.bmWidth; ++j)
	{
		//Convert to Full Opacity
		pPixel[3] = 255;
		
		//Remove Magic Pink
		if ( pPixel[0] == 255 && pPixel[1] == 0 && pPixel[2] == 255 )
		{
			pPixel[0] = 0;
			pPixel[1] = 0;
			pPixel[2] = 0;
			pPixel[3] = 0;
		}

		pPixel+= 4;
	}

	BLENDFUNCTION blendFunction = { AC_SRC_OVER, 0, alphaValue, AC_SRC_ALPHA };

	AlphaBlend(hDC, x, y, width, height, bufferDC, 0, 0, width, height, blendFunction);
		
	bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
	DeleteObject(bufferBitmap);
	DeleteDC(bufferDC);
}

void SolidTexture::solidGradientPaint(HDC destHDC, vector<COLORREF> workGradient, RECT r, int imageType)
{
	bool vertical = false;
	int repeated = 0;
	if ( imageType == 0 )
	{
		vertical = verticalGradient;
		repeated = repeatedGradient;
	}
	else if( imageType == 1 )
	{
		vertical = hoververticalGradient;
		repeated = repeatedHoverGradient;
	}
	else if( imageType == 2 )
	{
		vertical = pressedverticalGradient;
		repeated = repeatedPressedGradient;
	}
	
	if ( repeated == 0 )
	{
		int nWidth = r.right-r.left;
		int nHeight = r.bottom-r.top;

		RECT r2 = {0,0,0,0};
		double cr, cg, cb, dr, dg, db;

		// ColorsSteps between Start and End Color
		int gradSteps = 0;
		int unequalRest = 0;

		int colorCount = workGradient.size();

		if (vertical)
		{
			gradSteps = (nHeight-colorCount)/(colorCount-1);
			unequalRest = (nHeight-colorCount)%(colorCount-1);
		}
		else
		{
			gradSteps = (nWidth-colorCount)/(colorCount-1);
			unequalRest = (nWidth-colorCount)%(colorCount-1);
		}

		int lastGradientPos = 0;
		if (vertical)
			lastGradientPos = r.top;
		else
			lastGradientPos = r.left;

		int counter = 0;
		for (ColorRefIterator cit = workGradient.begin(); cit != workGradient.end(); cit++ )
		{
			if (cit == workGradient.end())
				break;

			counter++;

			COLORREF one = *cit;
			cit++;
			COLORREF two = *cit;
			cit--;
		
			// get current color
			cr = GetRValue( one );
			cg = GetGValue( one );
			cb = GetBValue( one );

			// Get delta color (+1 cause we must reach the Final Color)
			dr = (cr - GetRValue( two )) / (gradSteps+1);
			dg = (cg - GetGValue( two )) / (gradSteps+1);
			db = (cb - GetBValue( two )) / (gradSteps+1);

			//Paint Real Gradient Color
			//-------------------------------------------
			if ( unequalRest != 0 && counter > 1 )
			{
				unequalRest--;
				if (vertical)
				{
					SetRect(&r2, r.left, lastGradientPos, r.right, min(lastGradientPos+1, r.bottom));
				}
				else
				{
					SetRect(&r2, lastGradientPos, r.top, min(lastGradientPos+1, r.right), r.bottom);
				}

				if (r2.right-r2.left > 0 && r2.bottom-r2.top > 0)
				{
					::SetBkColor(destHDC, one);
					::ExtTextOut(destHDC, 0, 0, ETO_OPAQUE, &r2, NULL, 0, NULL);
				}
				
				if (vertical)
					lastGradientPos = r2.bottom;
				else
					lastGradientPos = r2.right;
			}
			//-------------------------------------------

			for (int i = 0; i <= gradSteps; i++)
			{
				if (vertical)
				{
					SetRect(&r2, r.left, lastGradientPos+i, r.right, min(lastGradientPos+i+1, r.bottom));
				}
				else
				{
					SetRect(&r2, lastGradientPos+i, r.top, min(lastGradientPos+i+1, r.right), r.bottom);
				}

				if (r2.right-r2.left > 0 && r2.bottom-r2.top > 0)
				{
					::SetBkColor(destHDC, RGB((int)cr, (int)cg, (int)cb));
					::ExtTextOut(destHDC, 0, 0, ETO_OPAQUE, &r2, NULL, 0, NULL);
				}
				
				cr -= dr;
				cg -= dg;
				cb -= db;
			}

			if (vertical)
				lastGradientPos = r2.bottom;
			else
				lastGradientPos = r2.right;
		}
	}
	else
	{
		int nWidth = r.right-r.left;
		int nHeight = r.bottom-r.top;

		RECT r2 = {0,0,0,0};
		COLORREF color;

		int lastGradientPos = 0;
		if (vertical)
			lastGradientPos = r.top;
		else
			lastGradientPos = r.left;

		int counter = 0;
		int nSize = 0;
		
		if (vertical)
			nSize = nHeight;
		else
			nSize = nWidth; 

		while ( counter < nSize )
		{
			for (ColorRefIterator cit = workGradient.begin(); cit != workGradient.end(); cit++ )
			{
				color = *cit;

				if (vertical)
				{
					SetRect(&r2, r.left, lastGradientPos, r.right, min(lastGradientPos+repeated, r.bottom));
				}
				else
				{
					SetRect(&r2, lastGradientPos, r.top, min(lastGradientPos+repeated, r.right), r.bottom);
				}

				if (r2.right-r2.left > 0 && r2.bottom-r2.top > 0 && color != RGB(255,0,255) )
				{
					::SetBkColor(destHDC, color);
					::ExtTextOut(destHDC, 0, 0, ETO_OPAQUE, &r2, NULL, 0, NULL);
				}

				if (vertical)
					lastGradientPos = r2.bottom;
				else
					lastGradientPos = r2.right;

				counter++;

				if ( counter == nSize )
					break;
			}
		}
	}
}
