/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2005 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#if !defined(__SOLIDTEXTURE_H)
#define __SOLIDTEXTURE_H

#include "Texture.h"

typedef vector<COLORREF>::iterator ColorRefIterator;

class SolidTexture : public Texture
{
public:

	SolidTexture();
	virtual ~SolidTexture();

	void configure(const string &uniqueKey, const string &groupKey, bool alphaMap);
	void apply(HDC hDC, int x, int y, int width, int height, int transparencyMode, int imageType);
	void applyOverlay(HDC hDC, int x, int y, int width, int height, int alphaValue, int imageType);
	void applyAnim(HDC hDC, int x, int y, int width, int height, int transparencyMode, int imageType, int count) { return; };

	bool isTransparencyNeeded() { return ( (backgroundColor == RGB(255,0,255) || hoverbackgroundColor == RGB(255,0,255) || pressedbackgroundColor == RGB(255,0,255)) ? true : false); };

private:

	void parseCFG(string cfg, COLORREF &bgcolor, COLORREF &lightcolor, COLORREF &darkcolor);
	void parseGradientCFG(string cfg, int imageType);
	void parseBevelCFG(string cfg, int &leftBevel, int &rightBevel, int &topBevel, int &bottomBevel);
	void solidGradientPaint(HDC destHDC, vector<COLORREF> workGradient, RECT r, int imageType);

	int bevelSizeLeft, bevelSizeRight, bevelSizeTop, bevelSizeBottom;
	int hoverbevelSizeLeft, hoverbevelSizeRight, hoverbevelSizeTop, hoverbevelSizeBottom;
	int pressedbevelSizeLeft, pressedbevelSizeRight, pressedbevelSizeTop, pressedbevelSizeBottom;

	bool verticalGradient;
	bool hoververticalGradient;
	bool pressedverticalGradient;
	
	int repeatedGradient;
	int repeatedHoverGradient;
	int repeatedPressedGradient;

	string gradientColor;
	COLORREF backgroundColor;
	COLORREF darkBevelColor;
	COLORREF lightBevelColor;

	string hovergradientColor;
	COLORREF hoverbackgroundColor;
	COLORREF hoverdarkBevelColor;
	COLORREF hoverlightBevelColor;

	string pressedgradientColor;
	COLORREF pressedbackgroundColor;
	COLORREF presseddarkBevelColor;
	COLORREF pressedlightBevelColor;

	vector<COLORREF> Gradient;
	vector<COLORREF> HoverGradient;
	vector<COLORREF> PressedGradient;
};

#endif
