/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2005 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "StdAfx.h"
#include "xFont.h"

//Text Alignment
//-----------------------------------------------------
#define ALIGN_LEFT 0
#define ALIGN_TOP 0
#define ALIGN_CENTER 1
#define ALIGN_RIGHT 2
#define ALIGN_BOTTOM 2
//-----------------------------------------------------

xFont::xFont()
{
	leftBorder = 0;
	rightBorder = 0;
	topBorder = 0;
	bottomBorder = 0;
}

xFont::~xFont()
{
}

void xFont::configure(const string &uniqueKey, const string &groupKey)
{
	NameValuePair CharSetConstants[] =	{
		{ "ANSI_CHARSET", ANSI_CHARSET },
		{ "BALTIC_CHARSET", BALTIC_CHARSET },
		{ "CHINESEBIG5_CHARSET", CHINESEBIG5_CHARSET },
		{ "DEFAULT_CHARSET", DEFAULT_CHARSET },
		{ "EASTEUROPE_CHARSET", EASTEUROPE_CHARSET },
		{ "GB2312_CHARSET", GB2312_CHARSET },
		{ "GREEK_CHARSET", GREEK_CHARSET },
		{ "HANGUL_CHARSET", HANGUL_CHARSET },
		{ "MAC_CHARSET", MAC_CHARSET },
		{ "RUSSIAN_CHARSET", RUSSIAN_CHARSET },
		{ "SHIFTJIS_CHARSET", SHIFTJIS_CHARSET },
		{ "SYMBOL_CHARSET", SYMBOL_CHARSET },
		{ "TURKISH_CHARSET", TURKISH_CHARSET },
		{ "VIETNAMESE_CHARSET", VIETNAMESE_CHARSET },
		{ "JOHAB_CHARSET", JOHAB_CHARSET },
		{ "ARABIC_CHARSET", ARABIC_CHARSET },
		{ "HEBREW_CHARSET", HEBREW_CHARSET },
		{ "THAI_CHARSET", THAI_CHARSET },
		{ "OEM_CHARSET", OEM_CHARSET },
		{ 0, 0 }
	};

	NameValuePair alignValues[] = {
		{ "left", DT_LEFT },
		{ "center", DT_CENTER },
		{ "right", DT_RIGHT },
		{ 0, 0 }
	};

	NameValuePair vertAlignValues[] = {
		{ "top", ALIGN_TOP },
		{ "center", ALIGN_CENTER },
		{ "bottom", ALIGN_BOTTOM },
		{ 0, 0 }
	};

	//Initialize Basic Helper Strings
	int numTokens = 0;
	string one = "";
	string two = "";
	string three = "";
	string four = "";
	string five = "";
	string six = "";
	string seven = "";

	shadow = Getx2RCBool(uniqueKey, groupKey, "FontShadow");
	shadowColor = Getx2RCColor(uniqueKey, groupKey, "FontShadowColor", RGB(128, 128, 128));
	shadowX = Getx2RCInt(uniqueKey, groupKey, "FontShadowX", 1, 0);
	shadowY = Getx2RCInt(uniqueKey, groupKey, "FontShadowY", 1, 0);

	textoutline = Getx2RCBool(uniqueKey, groupKey, "FontOutline");
	textoutlineColor = Getx2RCColor(uniqueKey, groupKey, "FontOutlineColor", RGB(128, 128, 128));

	name = Getx2RCString(uniqueKey, groupKey, "Font", "Arial");
	charSet = Getx2RCNamedValue(uniqueKey, groupKey, "FontCharSet", CharSetConstants, DEFAULT_CHARSET);
	height = Getx2RCInt(uniqueKey, groupKey, "FontHeight", 15, 6);
	color = Getx2RCColor(uniqueKey, groupKey, "FontColor", RGB(0, 0, 0));

	bold = Getx2RCBool(uniqueKey, groupKey, "FontBold");
	italic = Getx2RCBool(uniqueKey, groupKey, "FontItalic");
	underline = Getx2RCBool(uniqueKey, groupKey, "FontUnderline");

	smooth = Getx2RCBool(uniqueKey, groupKey, "FontSmoothing", true);
	if ( IsOS( OS_XP ) && smooth )
		cleartype = Getx2RCBool(uniqueKey, groupKey, "FontClearType", true);
	else
		cleartype = false;

	lineBreak = Getx2RCBool(uniqueKey, groupKey, "AutoLineBreak");
	newlineSpace = Getx2RCInt(uniqueKey, groupKey, "NewLineSpace", 0);

	//Borders
	string tmpBorders = Getx2RCLine(uniqueKey, groupKey, "Borders", "");
	if ( !tmpBorders.empty() )
	{
		numTokens = SplitString(tmpBorders, one, two, three, four, five, six, seven);

		if (numTokens == 1)
		{
			leftBorder = rightBorder = topBorder = bottomBorder = atoi(one.c_str());
		}
		else if (numTokens == 2)
		{
			leftBorder = rightBorder = atoi(one.c_str());
			topBorder = bottomBorder = atoi(two.c_str());
		}
		else if (numTokens == 4)
		{
			leftBorder = atoi(one.c_str());
			rightBorder = atoi(two.c_str());
			topBorder = atoi(three.c_str());
			bottomBorder = atoi(four.c_str());	
		}
		
		if ( leftBorder < 0 )
			leftBorder = 0;
		if ( rightBorder < 0 )
			rightBorder = 0;
		if ( topBorder < 0 )
			topBorder = 0;
		if ( bottomBorder < 0 )
			bottomBorder = 0;
	}

	if ( leftBorder == 0 )
		leftBorder = Getx2RCInt(uniqueKey, groupKey, "LeftBorder", 0, 0);
	if ( topBorder == 0 )
		topBorder = Getx2RCInt(uniqueKey, groupKey, "TopBorder", 0, 0);
	if ( rightBorder == 0 )
		rightBorder = Getx2RCInt(uniqueKey, groupKey, "RightBorder", 0, 0);
	if ( bottomBorder == 0 )
		bottomBorder = Getx2RCInt(uniqueKey, groupKey, "BottomBorder", 0, 0);

	//Alignment
	align = Getx2RCNamedValue(uniqueKey, groupKey, "FontAlign", alignValues, DT_CENTER);
	vertAlign = Getx2RCNamedValue(uniqueKey, groupKey, "FontVertAlign", vertAlignValues, ALIGN_CENTER);
	
	fntDefault.SetName(name.c_str());
	fntDefault.SetCharSet(charSet);
	fntDefault.SetSize(height);
	fntDefault.SetBold(bold);
	fntDefault.SetItalic(italic);
	fntDefault.SetUnderline(underline);
	fntDefault.SetSmooth(smooth);
	fntDefault.SetClearType(cleartype);
}

void xFont::apply(HDC hDC, int left, int right, int top, int bottom, const string &text, StringList &labelHREFClickRegions, bool smoothoff, int setalign, int setvertAlign)
{
	//Convert into MFC variables
	CDC tmpCDC;
	tmpCDC.Attach(hDC);

	CString strText = text.c_str();
	strText.Replace("<br><br>", "<br> <br>");
	strText.Replace("<br><p>", "<br> <p>");
	strText.Replace("<p><br>", "<p> <br>");
	strText.Replace("<p><p>", "<p> <p>");

	short nAlignment = align;
	short nVAlignment = vertAlign;

	if ( setalign != -1 )
		nAlignment = setalign;

	if ( setvertAlign != -1 )
		nVAlignment = setvertAlign;

	if ( smoothoff )
		fntDefault.SetSmooth(false);

	CRect rctPosition; 
	rctPosition.left = left;
	rctPosition.right = right;
	rctPosition.top = top;
	rctPosition.bottom = bottom;

	// Call to CHTMLDrawer
	if (shadow && !textoutline)
	{
		fntDefault.SetColor(shadowColor);

		CRect r;
		CopyRect(&r, &rctPosition);
		//Right
		r.OffsetRect(shadowX, shadowY);

		CHTMLDrawer::DrawText(&tmpCDC,
								strText,
								fntDefault,
								r,
								nAlignment,
								nVAlignment,
								lineBreak,
								newlineSpace,
								labelHREFClickRegions);
	}
	else if (textoutline)
	{
		fntDefault.SetColor(textoutlineColor);

		CRect r;
		CopyRect(&r, &rctPosition);
		//Right
		r.OffsetRect(1, 0);
		CHTMLDrawer::DrawText(&tmpCDC,
								strText,
								fntDefault,
								r,
								nAlignment,
								nVAlignment,
								lineBreak,
								newlineSpace,
								labelHREFClickRegions);
	
		//Left
		r.OffsetRect(-2, 0);
		CHTMLDrawer::DrawText(&tmpCDC,
								strText,
								fntDefault,
								r,
								nAlignment,
								nVAlignment,
								lineBreak,
								newlineSpace,
								labelHREFClickRegions);

		//(Center/)Bottom
		r.OffsetRect(1, -1);
		CHTMLDrawer::DrawText(&tmpCDC,
								strText,
								fntDefault,
								r,
								nAlignment,
								nVAlignment,
								lineBreak,
								newlineSpace,
								labelHREFClickRegions);

		//Top
		r.OffsetRect(0, 2);
		CHTMLDrawer::DrawText(&tmpCDC,
								strText,
								fntDefault,
								r,
								nAlignment,
								nVAlignment,
								lineBreak,
								newlineSpace,
								labelHREFClickRegions);
	}

	rctPosition.left = left;
	rctPosition.right = right;
	rctPosition.top = top;
	rctPosition.bottom = bottom;

	fntDefault.SetColor(color);

	CHTMLDrawer::DrawText(&tmpCDC,
							strText,
							fntDefault,
							rctPosition,
							nAlignment,
							nVAlignment,
							lineBreak,
							newlineSpace,
							labelHREFClickRegions);

	if ( smoothoff )
		fntDefault.SetSmooth(smooth);

	tmpCDC.Detach();
}

POINT xFont::measure(HDC hDC, int left, int right, const string &text)
{
	//Convert into MFC variables
	CDC tmpCDC;
	tmpCDC.Attach(hDC);

	CString strText = text.c_str();
	strText.Replace("<br><br>", "<br> <br>");
	strText.Replace("<br><p>", "<br> <p>");
	strText.Replace("<p><br>", "<p> <br>");
	strText.Replace("<p><p>", "<p> <p>");

	short nAlignment = align;
	short nVAlignment = vertAlign;

	CRect rctPosition; 
	if (lineBreak)
	{
		rctPosition.left = left;
		rctPosition.right = right;
	}
	else
	{
		rctPosition.left = 0;
		rctPosition.right = 0;
	}
	rctPosition.top = 0;
	rctPosition.bottom = 0;

	// Call to CHTMLDrawer
	POINT returnValues = CHTMLDrawer::MeasureText(&tmpCDC,
													strText,
													fntDefault,
													rctPosition,
													nAlignment,
													nVAlignment,
													lineBreak,
													newlineSpace);

	tmpCDC.Detach();
	
	return returnValues;
}

void xFont::setColor(int aColor, bool shadow)
{
	if (shadow)
		shadowColor = aColor;
	else
		color = aColor;
}

void xFont::setShadow(bool shadowEnabled)
{
	shadow = shadowEnabled;
}

HFONT xFont::gethFont()
{
	LOGFONT lf;
	memset(&lf, 0, sizeof(LOGFONT));

	StringCchCopy(lf.lfFaceName, LF_FACESIZE, name.c_str());
	lf.lfHeight = height;
	lf.lfItalic = italic ? TRUE : FALSE;
	lf.lfUnderline = underline ? TRUE : FALSE;
	lf.lfWeight = bold ? FW_BOLD : FW_NORMAL;
    lf.lfCharSet = charSet;

	return CreateFontIndirect(&lf);
}